Package: quantmod
Type: Package
Title: Quantitative Financial Modelling Framework
Version: 0.4.28
Authors@R: c(
  person(given=c("Jeffrey","A."), family="Ryan", role=c("aut","cph")),
  person(given=c("Joshua","M."), family="Ulrich", role=c("cre","aut"), email="josh.m.ulrich@gmail.com"),
  person(given=c("Ethan","B."), family="Smith", role="ctb"),
  person(given="Wouter", family="Thielen", role="ctb"),
  person(given="Paul", family="Teetor", role="ctb"),
  person(given="Steve", family="Bronder", role="ctb")
  )
Depends: R (>= 3.2.0), xts(>= 0.9-0), zoo, TTR(>= 0.2), methods
Imports: curl, jsonlite(>= 1.1)
Suggests: DBI, RMySQL, RSQLite, timeSeries, xml2, downloader, tinytest
Description: Specify, build, trade, and analyse quantitative financial trading strategies.
LazyLoad: yes
License: GPL-3
URL: https://www.quantmod.com/,
        https://github.com/joshuaulrich/quantmod
BugReports: https://github.com/joshuaulrich/quantmod/issues
NeedsCompilation: no
Packaged: 2025-06-18 21:49:58 UTC; josh
Author: Jeffrey A. Ryan [aut, cph],
  Joshua M. Ulrich [cre, aut],
  Ethan B. Smith [ctb],
  Wouter Thielen [ctb],
  Paul Teetor [ctb],
  Steve Bronder [ctb]
Maintainer: Joshua M. Ulrich <josh.m.ulrich@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-19 19:40:06 UTC
Built: R 4.6.0; ; 2026-04-26 02:23:39 UTC; unix
