`oanda.currencies` <-
structure(list(oanda.df.1.length.oanda.df...2....1. = c("US Dollar", 
"Afghanistan Afghani", "Albanian Lek", "Algerian Dinar", "Andorran Franc", 
"Andorran Peseta", "Angolan Kwanza", "Angolan New Kwanza", "Argentine Peso", 
"Armenian Dram", "Aruban Florin", "Australian Dollar", "Austrian Schilling", 
"Azerbaijan Manat", "Azerbaijan New Manat", "Bahamian Dollar", 
"Bahraini Dinar", "Bangladeshi Taka", "Barbados Dollar", "Belarusian Ruble", 
"Belgian Franc", "Belize Dollar", "Bermudian Dollar", "Bhutan Ngultrum", 
"Bolivian Boliviano", "Bosnian Mark", "Botswana Pula", "Brazilian Real", 
"British Pound", "Brunei Dollar", "Bulgarian Lev", "Burundi Franc", 
"CFA Franc BCEAO", "CFA Franc BEAC", "CFP Franc", "Cambodian Riel", 
"Canadian Dollar", "Cape Verde Escudo", "Cayman Islands Dollar", 
"Chilean Peso", "Chinese Yuan Renminbi", "Colombian Peso", "Comoros Franc", 
"Congolese Franc", "Costa Rican Colon", "Croatian Kuna", "Cuban Convertible Peso", 
"Cuban Peso", "Cyprus Pound", "Czech Koruna", "Danish Krone", 
"Djibouti Franc", "Dominican R. Peso", "Dutch Guilder", "ECU", 
"East Caribbean Dollar", "Ecuador Sucre", "Egyptian Pound", "El Salvador Colon", 
"Estonian Kroon", "Ethiopian Birr", "Euro", "Falkland Islands Pound", 
"Fiji Dollar", "Finnish Markka", "French Franc", "Gambian Dalasi", 
"Georgian Lari", "German Mark", "Ghanaian Cedi", "Ghanaian New Cedi", 
"Gibraltar Pound", "Gold (oz.)", "Greek Drachma", "Guatemalan Quetzal", 
"Guinea Franc", "Guyanese Dollar", "Haitian Gourde", "Honduran Lempira", 
"Hong Kong Dollar", "Hungarian Forint", "Iceland Krona", "Indian Rupee", 
"Indonesian Rupiah", "Iranian Rial", "Iraqi Dinar", "Irish Punt", 
"Israeli New Shekel", "Italian Lira", "Jamaican Dollar", "Japanese Yen", 
"Jordanian Dinar", "Kazakhstan Tenge", "Kenyan Shilling", "Kuwaiti Dinar", 
"Kyrgyzstanian Som", "Lao Kip", "Latvian Lats", "Lebanese Pound", 
"Lesotho Loti", "Liberian Dollar", "Libyan Dinar", "Lithuanian Litas", 
"Luxembourg Franc", "Macau Pataca", "Macedonian Denar", "Malagasy Ariary", 
"Malagasy Franc", "Malawi Kwacha", "Malaysian Ringgit", "Maldive Rufiyaa", 
"Maltese Lira", "Mauritanian Ouguiya", "Mauritius Rupee", "Mexican Peso", 
"Moldovan Leu", "Mongolian Tugrik", "Moroccan Dirham", "Mozambique Metical", 
"Mozambique New Metical", "Myanmar Kyat", "NL Antillian Guilder", 
"Namibia Dollar", "Nepalese Rupee", "New Zealand Dollar", "Nicaraguan Cordoba Oro", 
"Nigerian Naira", "North Korean Won", "Norwegian Kroner", "Omani Rial", 
"Pakistan Rupee", "Palladium (oz.)", "Panamanian Balboa", "Papua New Guinea Kina", 
"Paraguay Guarani", "Peruvian Nuevo Sol", "Philippine Peso", 
"Platinum (oz.)", "Polish Zloty", "Portuguese Escudo", "Qatari Rial", 
"Romanian Lei", "Romanian New Lei", "Russian Rouble", "Rwandan Franc", 
"Samoan Tala", "Sao Tome/Principe Dobra", "Saudi Riyal", "Serbian Dinar", 
"Seychelles Rupee", "Sierra Leone Leone", "Silver (oz.)", "Singapore Dollar", 
"Slovak Koruna", "Slovenian Tolar", "Solomon Islands Dollar", 
"Somali Shilling", "South African Rand", "South-Korean Won", 
"Spanish Peseta", "Sri Lanka Rupee", "St. Helena Pound", "Sudanese Dinar", 
"Sudanese Old Pound", "Sudanese Pound", "Suriname Dollar", "Suriname Guilder", 
"Swaziland Lilangeni", "Swedish Krona", "Swiss Franc", "Syrian Pound", 
"Taiwan Dollar", "Tanzanian Shilling", "Thai Baht", "Tonga Pa'anga", 
"Trinidad/Tobago Dollar", "Tunisian Dinar", "Turkish Lira", "Turkish New Lira", 
"Turkmenistan Manat", "Uganda Shilling", "Ukraine Hryvnia", "Uruguayan Peso", 
"Utd. Arab Emir. Dirham", "Vanuatu Vatu", "Venezuelan Bolivar", 
"Vietnamese Dong", "Yemeni Rial", "Yugoslav Dinar", "Zambian Kwacha", 
"Zimbabwe Dollar")), .Names = "oanda.df.1.length.oanda.df...2....1.", row.names = c("USD", 
"AFN", "ALL", "DZD", "ADF", "ADP", "AOA", "AON", "ARS", "AMD", 
"AWG", "AUD", "ATS", "AZM", "AZN", "BSD", "BHD", "BDT", "BBD", 
"BYR", "BEF", "BZD", "BMD", "BTN", "BOB", "BAM", "BWP", "BRL", 
"GBP", "BND", "BGN", "BIF", "XOF", "XAF", "XPF", "KHR", "CAD", 
"CVE", "KYD", "CLP", "CNY", "COP", "KMF", "CDF", "CRC", "HRK", 
"CUC", "CUP", "CYP", "CZK", "DKK", "DJF", "DOP", "NLG", "XEU", 
"XCD", "ECS", "EGP", "SVC", "EEK", "ETB", "EUR", "FKP", "FJD", 
"FIM", "FRF", "GMD", "GEL", "DEM", "GHC", "GHS", "GIP", "XAU", 
"GRD", "GTQ", "GNF", "GYD", "HTG", "HNL", "HKD", "HUF", "ISK", 
"INR", "IDR", "IRR", "IQD", "IEP", "ILS", "ITL", "JMD", "JPY", 
"JOD", "KZT", "KES", "KWD", "KGS", "LAK", "LVL", "LBP", "LSL", 
"LRD", "LYD", "LTL", "LUF", "MOP", "MKD", "MGA", "MGF", "MWK", 
"MYR", "MVR", "MTL", "MRO", "MUR", "MXN", "MDL", "MNT", "MAD", 
"MZM", "MZN", "MMK", "ANG", "NAD", "NPR", "NZD", "NIO", "NGN", 
"KPW", "NOK", "OMR", "PKR", "XPD", "PAB", "PGK", "PYG", "PEN", 
"PHP", "XPT", "PLN", "PTE", "QAR", "ROL", "RON", "RUB", "RWF", 
"WST", "STD", "SAR", "RSD", "SCR", "SLL", "XAG", "SGD", "SKK", 
"SIT", "SBD", "SOS", "ZAR", "KRW", "ESP", "LKR", "SHP", "SDD", 
"SDP", "SDG", "SRD", "SRG", "SZL", "SEK", "CHF", "SYP", "TWD", 
"TZS", "THB", "TOP", "TTD", "TND", "TRL", "TRY", "TMM", "UGX", 
"UAH", "UYU", "AED", "VUV", "VEB", "VND", "YER", "YUN", "ZMK", 
"ZWD"), class = "data.frame")
